"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Highlighter = void 0;
// third party
const chalk_1 = __importDefault(require("chalk"));
const prismjs_1 = __importDefault(require("prismjs"));
require("prismjs/components/prism-json");
require("prismjs/components/prism-markdown");
require("prismjs/components/prism-graphql");
const jsdom_1 = require("jsdom");
// locals
const themes_1 = require("./themes");
/**
 * terminal code highlighter imported with very little modification from here:
 * https://github.com/yukihirop/rue/blob/main/packages/generator/src/definitions/highlighter/
 */
class Highlighter {
    static highlight(sourceCode, sourceLang) {
        const prismCode = prismjs_1.default.highlight(sourceCode, prismjs_1.default.languages[sourceLang], sourceLang);
        const dom = jsdom_1.JSDOM.fragment(prismCode);
        const highlightedSource = this._parseFormatedContent(dom.childNodes, 0);
        return highlightedSource;
    }
    // https://medium.com/@stfbauer/use-prismjs-for-syntax-highlighting-in-console-log-in-nodejs-ab84b6eaa079#
    static _parseFormatedContent(domElement, recLevel) {
        let highlightedSource = '';
        domElement.forEach((element) => {
            if (element.hasChildNodes()) {
                // @ts-ignore
                const highlighter = this._getHighlighter(element.classList);
                if (highlighter) {
                    highlightedSource += highlighter(this._parseFormatedContent(element.childNodes, recLevel + 1));
                }
                else {
                    highlightedSource += element.textContent;
                }
            }
            else {
                highlightedSource += element.textContent;
            }
        });
        return highlightedSource;
    }
    static _getHighlighter(tokens) {
        const supportTokens = this.SUPPORT_TOKENS;
        let highliter;
        for (let i = 0; i < tokens.length; i++) {
            if (supportTokens.indexOf(tokens[i]) !== -1) {
                const token = tokens[i];
                highliter = chalk_1.default.hex(themes_1.prismOkaidia[token]);
                break;
            }
        }
        if (highliter) {
            return highliter;
        }
        else {
            return (code) => code;
        }
    }
}
exports.Highlighter = Highlighter;
Highlighter.SUPPORT_TOKENS = Object.keys(themes_1.prismOkaidia);
